package pico.orange.ide.console;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.FocusEvent;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.text.Caret;
import javax.swing.text.DefaultCaret;

public class Console extends JPanel {

	private JTextArea text;
	private final Font defaultEditFont = new Font("ＭＳ ゴシック", Font.PLAIN, 16);
	private JScrollPane scroll;
	ByteArrayOutputStream bytes;

	/**
	 * コンソールパネルを構築します。
	 */
	public Console() {
		super(new BorderLayout());
		this.text = new JTextArea();
		this.text.setBackground(Color.WHITE);
		this.text.setForeground(Color.BLACK);
		this.text.setCaretColor(Color.BLACK);
		this.text.setFont(this.defaultEditFont);
		this.scroll = new JScrollPane(this.text, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		this.add(this.scroll);

		this.text.setEditable(false);
		Caret c = this.text.getCaret();
		Caret nc = new CustomCaret(c.getBlinkRate());
		this.text.setCaret(nc);

		this.bytes = new ByteArrayOutputStream() {
			@Override
			public synchronized void flush() throws IOException {
				Console.this.text.setText(this.toString());
			}
		};
		PrintStream out = new PrintStream(this.bytes, true);
		System.setErr(out);
		System.setOut(out);
	}

	public void cls() {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				Console.this.text.setText("");
				Console.this.bytes.reset();
			}
		});
	}

	class CustomCaret extends DefaultCaret {
		public CustomCaret(int rate) {
			this.setBlinkRate(rate);
		}

		@Override
		public void focusGained(FocusEvent e) {
			this.setVisible(true);
			this.setSelectionVisible(true);
		}
	}
}
