package pico.orange.ide.editor;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;

import javax.swing.SizeRequirements;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.ParagraphView;
import javax.swing.text.Position;

import pico.orange.ide.Config;

public class CustomParagraphView extends ParagraphView {
	public CustomParagraphView(Element elem) {
		super(elem);
	}

	// 行を折り返さない
	@Override
	protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements r) {
		SizeRequirements req = super.calculateMinorAxisRequirements(axis, r);
		req.minimum = req.preferred;
		return req;
	}

	@Override
	public int getFlowSpan(int index) {
		return Integer.MAX_VALUE;
	}

	@Override
	public void paint(Graphics g, Shape a) {
		super.paint(g, a);
		if (this.getEndOffset() != this.getDocument().getEndPosition().getOffset()) { // 最後は改行マークを表示しない
			this.paintOriginalParagraph(g, a);
		}
	}

	/**
	 * 改行マークを表示します。
	 *
	 * @param g
	 * @param a
	 */
	private void paintOriginalParagraph(Graphics g, Shape a) {
		try {
			Shape paragraph = this.modelToView(this.getEndOffset(), a, Position.Bias.Backward);
			Rectangle rec = (paragraph == null) ? a.getBounds() : paragraph.getBounds();
			int X = rec.x;
			int Y = rec.y;
			int HEIGHT = rec.height;
			int WIDTH = HEIGHT / 2;
			Color defaultColor = g.getColor();

			g.setColor(Config.LF_COLOR);
			g.drawLine(X + 1, Y + HEIGHT - 4, X + WIDTH - 4, Y + HEIGHT - 4);
			g.drawLine(X + 1, Y + HEIGHT - 4, X + WIDTH / 2 - 2, Y + HEIGHT * 3 / 4);
			g.drawLine(X + WIDTH - 4, Y + HEIGHT / 2, X + WIDTH - 4, Y + HEIGHT - 4);

			g.setColor(defaultColor);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}
}
