package pico.orange.ide.editor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;

import pico.orange.ide.Config;

class LineNumberView extends JComponent {

	private static int MARGIN = 5;
	private JTextComponent text;
	private FontMetrics fontMetrics;
	private int topInset;
	private int fontAscent;
	private int fontHeight;
	private int fontDescent;
	private int fontLeading;

	public LineNumberView(JTextComponent text) {
		super();
		this.text = text;
		Font font = text.getFont();
		this.fontMetrics = this.getFontMetrics(font);
		this.fontHeight = this.fontMetrics.getHeight();
		this.fontAscent = this.fontMetrics.getAscent();
		this.fontDescent = this.fontMetrics.getDescent();
		this.fontLeading = this.fontMetrics.getLeading();
		this.topInset = text.getInsets().top;

		text.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				LineNumberView.this.repaint();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				LineNumberView.this.repaint();
			}

			@Override
			public void changedUpdate(DocumentEvent e) {
			}
		});

		text.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				LineNumberView.this.revalidate();
				LineNumberView.this.repaint();
			}
		});

		Insets i = text.getInsets();
		this.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 0, 1, Color.WHITE),
				BorderFactory.createEmptyBorder(i.top, LineNumberView.MARGIN, i.bottom, LineNumberView.MARGIN - 1)));
		this.setOpaque(true);
		this.setForeground(Config.LINE_NUMBER_FG_COLOR);
		this.setBackground(Config.LINE_NUMBER_BG_COLOR);
		this.setFont(font);
	}

	private int getComponentWidth() {
		Document doc = this.text.getDocument();
		Element root = doc.getDefaultRootElement();
		int lineCount = root.getElementIndex(doc.getLength());
		int maxDigits = Math.max(3, String.valueOf(lineCount).length());
		Insets i = this.getBorder().getBorderInsets(this);
		return maxDigits * this.fontMetrics.stringWidth("0") + i.left + i.right;
	}

	private int getLineAtPoint(int y) {
		Element root = this.text.getDocument().getDefaultRootElement();
		int pos = this.text.viewToModel(new Point(0, y));
		return root.getElementIndex(pos);
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(this.getComponentWidth(), this.text.getHeight());
	}

	@Override
	public void paintComponent(Graphics g) {
		g.setColor(this.getBackground());
		Rectangle clip = g.getClipBounds();
		g.fillRect(clip.x, clip.y, clip.width, clip.height);
		g.setColor(this.getForeground());
		int base = clip.y;
		int start = this.getLineAtPoint(base);
		int end = this.getLineAtPoint(base + clip.height);
		int y = start * this.fontHeight + this.topInset;
		int rmg = this.getBorder().getBorderInsets(this).right;
		for (int i = start; i <= end; i++) {
			String text = String.format("%3d", i + 1);
			int x = this.getComponentWidth() - rmg - this.fontMetrics.stringWidth(text);
			y += this.fontAscent;
			g.drawString(text, x, y);
			y += this.fontDescent + this.fontLeading;
		}
	}
}
