package pico.orange.ide.editor;

import java.awt.Color;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Element;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

public class SyntaxDocument extends DefaultStyledDocument {

	static String[] keyList = {
		"org",
		"dn",
		"ink",
		"outn",
		"abyz",
		"ay",
		"st",
		"ld",
		"add",
		"sub",
		"ldi",
		"addi",
		"ldyi",
		"addyi",
		"cpi",
		"cpyi",
		"scall",
		"jmpf",
		"call",
		"ret",
		"pusha",
		"popa",
		"pushb",
		"popb",
		"pushy",
		"popy",
		"pushz",
		"popz",
		"ioctrl",
		"out",
		"in"
	};
	private Style normal;
	private static final String OPERANDS = "(,";

	public SyntaxDocument() {
		super();
		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
		this.normal = this.addStyle("normal", def);
		StyleConstants.setForeground(this.normal, Color.BLACK); // エディターの文字色
		for (String key : SyntaxDocument.keyList) {
			StyleConstants.setForeground(this.addStyle(key, this.normal), Color.BLUE);
		}

	}

	@Override
	public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
		super.insertString(offset, str, a);
		this.processChangedLines(offset, str.length());
	}

	@Override
	public void remove(int offset, int length) throws BadLocationException {
		super.remove(offset, length);
		this.processChangedLines(offset, 0);
	}

	private void processChangedLines(int offset, int length) throws BadLocationException {
		Element root = this.getDefaultRootElement();
		String content = this.getText(0, this.getLength());
		int startLine = root.getElementIndex(offset);
		int endLine = root.getElementIndex(offset + length);
		for (int i = startLine; i <= endLine; i++) {
			this.applyHighlighting(content, i);
		}
	}

	private void applyHighlighting(String content, int line) throws BadLocationException {
		Element root = this.getDefaultRootElement();
		int startOffset = root.getElement(line).getStartOffset();
		int endOffset = root.getElement(line).getEndOffset() - 1;
		int lineLength = endOffset - startOffset;
		int contentLength = content.length();
		if (endOffset >= contentLength) {
			endOffset = contentLength - 1;
		}
		this.setCharacterAttributes(startOffset, lineLength, this.normal, true);
		this.checkForTokens(content, startOffset, endOffset);
	}

	private void checkForTokens(String content, int startOffset, int endOffset) {
		int index = startOffset;
		while (index <= endOffset) {
			while (this.isDelimiter(content.substring(index, index + 1))) {
				if (index < endOffset) {
					index++;
				} else {
					return;
				}
			}
			index = this.getOtherToken(content, index, endOffset);
		}
	}

	private int getOtherToken(String content, int startOffset, int endOffset) {
		int endOfToken = startOffset + 1;
		while (endOfToken <= endOffset) {
			if (this.isDelimiter(content.substring(endOfToken, endOfToken + 1))) {
				break;
			}
			endOfToken++;
		}
		String token = content.substring(startOffset, endOfToken);
		Style s = this.getStyle(token);
		if (s != null) {
			this.setCharacterAttributes(startOffset, endOfToken - startOffset, s, false);
		}
		return endOfToken + 1;
	}

	protected boolean isDelimiter(String character) {
		return Character.isWhitespace(character.charAt(0)) || SyntaxDocument.OPERANDS.indexOf(character) != -1;
	}

}