package pico.orange.compiler;

import java.util.HashMap;
import java.util.Map;

import pico.orange.compiler.Token.TokenValue;

public class Assembler {

	private Map<String, Integer> labelTable = new HashMap<String, Integer>();
	private boolean isError = false;

	/**
	 *
	 * @param src
	 */
	public void assemble(String src) {
		if (src.equals("")) {
			return;
		}
		Lexer lexer = new Lexer(src);

		// パス1
		int pc = 0;
		Token token = lexer.nextToken();
		while (true) {
			TokenValue tokenValue = token.getTokenValue();

			// String debug = String.format("%05d:%s", token.getLine(),
			// tokenValue);
			// System.out.println(debug);
			switch (tokenValue) {
			case ORG:
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					pc = token.getIntValue();
					token = lexer.nextToken();
				} else {
					this.error(token, "ORG: オペランド不正");
					return;
				}
				break;
			case IDENT:
				String key = token.getStringValue();
				if (lexer.lookToken(1).getTokenValue() == TokenValue.COLON) {
					this.labelTable.put(key, pc); // テーブルに格納
				}
				token = lexer.nextToken();
				break;
			case INK:
			case OUTN:
			case ABYZ:
			case AY:
			case ST:
			case LD:
			case ADD:
			case SUB:
				token = lexer.nextToken();
				pc++;
				break;
			case LDI:
			case ADDI:
			case LDYI:
			case ADDYI:
			case CPI:
			case CPYI:
			case SCALL:
				token = lexer.nextToken();
				pc += 2;
				break;
			case JMPF:
				token = lexer.nextToken();
				pc += 3;
				break;
			case CALL:
				token = lexer.nextToken();
				pc += 5;
				break;
			case RET:
			case PUSHA:
			case POPA:
			case PUSHB:
			case POPB:
			case PUSHY:
			case POPY:
			case PUSHZ:
			case POPZ:
			case IOCTRL:
			case OUT:
			case IN:
				token = lexer.nextToken();
				pc += 3;
				break;
			default:
				token = lexer.nextToken();
				break;
			}
			if (tokenValue == TokenValue.EOF) {
				break;
			}
		}

		// パス2
		lexer = new Lexer(src);
		pc = 0;
		int lastPc = 0;
		boolean isOrg = false;
		StringBuilder buf = new StringBuilder();
		while (true) {
			token = lexer.nextToken();
			TokenValue tokenValue = token.getTokenValue();
			// String debug = String.format("%05d:%s", token.getLine(),
			// tokenValue);
			// System.out.println(debug);
			switch (tokenValue) {
			case ORG:
				token = lexer.nextToken();
				pc = token.getIntValue();
				if (isOrg && !this.isError) {
					String s = String.format("E%02X:%s", lastPc, buf.toString());
					System.out.println(s);
					buf = new StringBuilder();
				}
				lastPc = pc;
				isOrg = true;
				break;
			case DN:
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "DN: オペランド不正");
				}
				break;
			case INK:
				buf.append('0');
				break;
			case OUTN:
				buf.append('1');
				break;
			case ABYZ:
				buf.append('2');
				break;
			case AY:
				buf.append('3');
				break;
			case ST:
				buf.append('4');
				break;
			case LD:
				buf.append('5');
				break;
			case ADD:
				buf.append('6');
				break;
			case SUB:
				buf.append('7');
				break;
			case LDI:
				buf.append('8');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "LDI: オペランド不正");
				}
				break;
			case ADDI:
				buf.append('9');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "ADDI: オペランド不正");
				}
				break;
			case LDYI:
				buf.append('A');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "LDYI: オペランド不正");
				}
				break;
			case ADDYI:
				buf.append('B');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "ADDYI: オペランド不正");
				}
				break;
			case CPI:
				buf.append('C');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "CPI: オペランド不正");
				}
				break;
			case CPYI:
				buf.append('D');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "CPYI: オペランド不正");
				}
				break;
			case SCALL:
				buf.append('E');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%X", token.getIntValue() & 0xf));
				} else {
					this.error(token, "LDI: オペランド不正");
				}
				break;
			case JMPF:
				buf.append('F');
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%02x", token.getIntValue() & 0xff));
				} else if (tokenValue == TokenValue.IDENT) {
					String key = token.getStringValue();
					Integer v = this.labelTable.get(key);
					if (v == null) {
						this.error(token, "JMPF: ラベルが見つからない");
					} else {
						buf.append(String.format("%02X", v & 0xff));
					}
				} else {
					this.error(token, "JMPF: オペランド不正");
				}
				break;
			case CALL:
				buf.append("F60");
				token = lexer.nextToken();
				tokenValue = token.getTokenValue();
				if (tokenValue == TokenValue.LITERAL) {
					buf.append(String.format("%02x", token.getIntValue() & 0xff));
				} else if (tokenValue == TokenValue.IDENT) {
					String key = token.getStringValue();
					Integer v = this.labelTable.get(key);
					if (v == null) {
						this.error(token, "CALL: ラベルが見つからない");
					} else {
						buf.append(String.format("%02X", v & 0xff));
					}
				} else {
					this.error(token, "CALL: Illegal operand\n");
				}
				break;
			case RET:
				buf.append("F61");
				break;
			case PUSHA:
				buf.append("F62");
				break;
			case POPA:
				buf.append("F63");
				break;
			case PUSHB:
				buf.append("F64");
				break;
			case POPB:
				buf.append("F65");
				break;
			case PUSHY:
				buf.append("F66");
				break;
			case POPY:
				buf.append("F67");
				break;
			case PUSHZ:
				buf.append("F68");
				break;
			case POPZ:
				buf.append("F69");
				break;
			case IOCTRL:
				buf.append("F70");
				break;
			case OUT:
				buf.append("F71");
				break;
			case IN:
				buf.append("F72");
				break;
			default:
				break;
			}
			if (tokenValue == TokenValue.EOF) {
				break;
			}
		}
		if (this.isError) {
			return;
		} else {
			String s = buf.toString();
			if (!s.equals("")) {
				s = String.format("E%02X:%s", pc, s);
				System.out.println(s);
			}
		}
	}

	void error(Token token, String message) {
		String s = String.format("LINE = %03d: %s", token.getLine(), message);
		System.err.println(s);
		this.isError = true;
	}

}
